@@ -46,11 +46,39 @@ class GithubBackend(OAuthBackend):
         ('expires', 'expires')
     ]
 
+    def _fetch_emails(self, access_token):
+        """Fetch private emails from Github account"""
+        url = GITHUB_USER_DATA_URL + '/emails?' + urlencode({
+            'access_token': access_token
+        })
+
+        try:
+            data = simplejson.load(dsa_urlopen(url))
+        except (ValueError, HTTPError):
+            data = []
+        return data
+
     def get_user_details(self, response):
         """Return user details from Github account"""
-        return {'username': response.get('login'),
-                'email': response.get('email') or '',
-                'first_name': response.get('name') or ''}
+        name = response.get('name') or ''
+        details = {'username': response.get('login')}
+
+        try:
+            email = self._fetch_emails(response.get('access_token'))[0]
+        except IndexError:
+            details['email'] = ''
+        else:
+            details['email'] = email
+
+        try:
+            # GitHub doesn't separate first and last names. Let's try.
+            first_name, last_name = name.split(' ', 1)
+        except ValueError:
+            details['first_name'] = name
+        else:
+            details['first_name'] = first_name
+            details['last_name'] = last_name
+        return details
 
 
 class GithubAuth(BaseOAuth2):
