@@ -1,3 +1,4 @@
+from django.utils.translation import get_language
 """
 Linkedin OAuth support
 
@@ -6,10 +7,14 @@ No extra configurations are needed to make this work.
 from xml.etree import ElementTree
 from xml.parsers.expat import ExpatError
 
+from urllib import urlencode
+from urllib2 import Request
 from oauth2 import Token
 
-from social_auth.utils import setting
-from social_auth.backends import ConsumerBasedOAuth, OAuthBackend
+from django.utils import simplejson
+
+from social_auth.utils import setting, dsa_urlopen
+from social_auth.backends import ConsumerBasedOAuth, OAuthBackend, BaseOAuth2
 from social_auth.exceptions import AuthCanceled, AuthUnknownError
 
 
@@ -26,6 +31,14 @@ LINKEDIN_CHECK_AUTH = 'https://api.%s/v1/people/~' % LINKEDIN_SERVER
 LINKEDIN_FIELD_SELECTORS = ['id', 'first-name', 'last-name']
 
 
+def add_language_header(request):
+    language = setting('LINKEDIN_FORCE_PROFILE_LANGUAGE', False)
+    if language is True:
+        request.add_header('Accept-Language', get_language())
+    elif language:
+        request.add_header('Accept-Language', language)
+
+
 class LinkedinBackend(OAuthBackend):
     """Linkedin OAuth authentication backend"""
     name = 'linkedin'
@@ -43,6 +56,32 @@ class LinkedinBackend(OAuthBackend):
                 'last_name': last_name,
                 'email': email}
 
+    @classmethod
+    def tokens(cls, instance):
+        """ Return list of OAuth v1 tokens from Linkedin  """
+        token = super(LinkedinBackend, cls).tokens(instance)
+        if token and 'access_token' in token:
+            token = dict(tok.split('=')
+                            for tok in token['access_token'].split('&'))
+        return token
+
+
+class LinkedinOAuth2Backend(OAuthBackend):
+    """Linkedin OAuth2 authentication backend"""
+    name = 'linkedin-oauth2'
+
+    EXTRA_DATA = [('id', 'id'),
+                  ('firstName', 'first_name'),
+                  ('lastName', 'last_name')]
+
+    def get_user_details(self, response):
+        first_name, last_name = response['firstName'], response['lastName']
+        return {'username': first_name + last_name,
+                'fullname': first_name + ' ' + last_name,
+                'first_name': first_name,
+                'last_name': last_name,
+                'email': response.get('emailAddress', '')}
+
 
 class LinkedinAuth(ConsumerBasedOAuth):
     """Linkedin OAuth authentication mechanism"""
@@ -63,6 +102,7 @@ class LinkedinAuth(ConsumerBasedOAuth):
         # duplicated
         url = LINKEDIN_CHECK_AUTH + ':(%s)' % ','.join(set(fields_selectors))
         request = self.oauth_request(access_token, url)
+        add_language_header(request)
         raw_xml = self.fetch_response(request)
         try:
             return to_dict(ElementTree.fromstring(raw_xml))
@@ -106,6 +146,29 @@ class LinkedinAuth(ConsumerBasedOAuth):
         return Token.from_string(response)
 
 
+class LinkedinOAuth2(BaseOAuth2):
+    AUTH_BACKEND = LinkedinOAuth2Backend
+    AUTHORIZATION_URL = 'https://www.linkedin.com/uas/oauth2/authorization'
+    ACCESS_TOKEN_URL = 'https://www.linkedin.com/uas/oauth2/accessToken'
+    SETTINGS_KEY_NAME = 'LINKEDIN_CONSUMER_KEY'
+    SETTINGS_SECRET_NAME = 'LINKEDIN_CONSUMER_SECRET'
+    SCOPE_VAR_NAME = 'LINKEDIN_SCOPE'
+    REDIRECT_STATE = False
+
+    def user_data(self, access_token, *args, **kwargs):
+        """Return user data provided"""
+        fields_selectors = LINKEDIN_FIELD_SELECTORS + \
+                           setting('LINKEDIN_EXTRA_FIELD_SELECTORS', [])
+        url = LINKEDIN_CHECK_AUTH + ':(%s)' % ','.join(set(fields_selectors))
+        data = {'oauth2_access_token': access_token, 'format': 'json'}
+        request = Request(url + '?' + urlencode(data))
+        add_language_header(request)
+        try:
+            return simplejson.loads(dsa_urlopen(request).read())
+        except (ExpatError, KeyError, IndexError):
+            return None
+
+
 def to_dict(xml):
     """Convert XML structure to dict recursively, repeated keys entries
     are returned as in list containers."""
@@ -127,4 +190,5 @@ def to_dict(xml):
 # Backend definition
 BACKENDS = {
     'linkedin': LinkedinAuth,
+    'linkedin-oauth2': LinkedinOAuth2,
 }
